def ignore_errors(function):
    """
    Simple decorator, note the nesting.
    :param function: The function to ne decorated.
    :return: The decorated function
    """
    def wrapper(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except:
            pass

    return wrapper


@ignore_errors
def divide(a, b):
    return a / b


print(divide(5, 0))


def ignore_errors_return(def_value):
    """
    Note the two nested functions. This is necessary to make it work.
    :param def_value: The default value to be returned in case of an exception.
    :return: The function decorator (wrapper)
    """

    def wrapper(function):
        def checked_division(*args, **kwargs):
            try:
                return function(*args, **kwargs)
            except:
                return def_value

        return checked_division

    return wrapper


@ignore_errors_return(def_value=0)
def divide2(a, b):
    return a / b


print(divide2(5, 0))


# Class decorator #

def public_methods_decorator(class_):
    """
    This method is called during constructor call. It iterates over all elements of the class and for those that are
    callable and whose name does not start '_' it adds the wrapper.
    :param class_: The class object to be decorated
    :return: The decorated class, i.e., the class with replaced (decorated) public methods
    """
    def wrapper(method_):
        def intr(*args, **kwargs):
            print('Method entry ' + method_.__name__)
            res = method_(*args, **kwargs)
            print('Method exit ' + method_.__name__)
            return res

        return intr

    #  The following code adds the wrapper to public methods
    for attr, val in class_.__dict__.items():
        if callable(val) and not attr.startswith('_'):
            setattr(class_, attr, wrapper(val))

    return class_


@public_methods_decorator
class Foo:
    att = 0

    def __init__(self):
        self.att = 0

    def public_m1(self):
        print('m1')

    def public_m2(self):
        self.public_m1()
        self._private_m()

    def _private_m(self):
        pass


foo = Foo()

foo.public_m1()
foo.public_m2()
